==> jmode-0.4.20.tar.gz: jmode-0.4.20/src/cannaconv.cpp <==
// libcannaconv.so canna Ѥplugin
//
#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <canna/jrkanji.h>
#include "canna.xpm"
#include "cannaconv.h"

bool init_conv()
{
  // ⤷ǼԤ顢δؿfalse֤
  CannaConv *canna = new CannaConv();
  canna->init();
  register_kkconv(canna);
    
  return true;
}

CannaConv::~CannaConv()
{
  // λν
  jrKanjiControl(0,KC_FINALIZE,NULL);
}

KKContext *CannaConv::createContext(ximic *ic)
{
  // ϥƥȤ롣
  CannaContext *canna;
  canna = new CannaContext(this,ic);
  return canna;
}

char **CannaConv::getIcon()
{
  // toolbar˽ФƤ륢֤
  return canna_xpm;
}

void CannaConv::onPushIcon()
{
  // toolbarβ줿˸ƤФ롣
}

void CannaConv::init()
{
  //Cannaץꥱ̾Ρ⡼ɾͤɽ
  jrKanjiControl(0,KC_INITIALIZE,NULL);
  jrKanjiControl(0,KC_SETAPPNAME,"HekeIM");
  jrKanjiControl(0,KC_SETMODEINFOSTYLE,(char *)1);  
}

/////////////////////////////////////
//
CannaContext::CannaContext(CannaConv *,ximic *ic) : KKContext(ic)
{
  contextID = (int)this;
  
  ksv.val = CANNA_MODE_AlphaMode;
  ksv.buffer = (unsigned char *)buffer;
  ksv.bytes_buffer = BUF_SIZE;
  ksv.ks = &ks;

  jrKanjiControl(contextID,KC_INITIALIZE,0);
  //jrKanjiControl(contextID,KC_SETMODEINFOSTYLE,(char*)1);
  mode = INPUT_MODE_RAW;
}

CannaContext::~CannaContext()
{
  // ȤʤʤäƥȤ
  jrKanjiControl(contextID,KC_CLOSEUICONTEXT,(char*)&ksv);
}

void CannaContext::OnUpdatePe(pe_stat *pe)
{
  // preedit˸ƤФ롣
  pe->clear();
  /* 
     if ( mode != INPUD_MODE_HIRA && mode!= INPUT_MODE_KATA){
     return ;// Ѵ⡼ɤǤʤpreedit¸ߤʤ
     }
     pe->new_segment(PE_UNDERLINE);//饢饤
     jstring_t s;
     p = ߤΥץꥨǥåʸ
     str_to_jstring(buffer,&s);
     jstring_t::iterator i;
     for ( i = s.begin() ; i != s.end() ; i++){
     pe->push_cchar(*i);
     }
     */
}

int CannaContext::push_key(keyState *e)
{
  // 줿Τˡ
  if ( mode == INPUT_MODE_RAW || mode == INPUT_MODE_WIDE){
    //줿Τ⡼ڤؤʳä餽Τޤcommitؿ
    return proc_raw_mode(e);
  }
  int nbytes;
  //Ѥʿ̾Ҳ̾ʤΤǤcannaΥȥޥȥ
  nbytes = jrKanjiString(contextID,e->char_code(),buffer,BUF_SIZE,&ks);
  //ץꥨǥåȤξ֤ѲƤΤǹ
  if ( nbytes > 0){
    buffer[nbytes] = 0;
    //update_preedit();
    printf("%s.\n",buffer);
  }
  return COMMIT_RAW;
}

int CannaContext::proc_raw_mode(keyState *e)
{
  if ( e->is_bind_to(A_hira_mode)){
    set_mode(INPUT_MODE_HIRA);
    return UPDATE_MODE;
  }
  return COMMIT_RAW;
}

int CannaContext::get_mode()
{
  return mode;
}

void CannaContext::set_mode(int m)
{
  if ( mode == m){
    return ;
  }

  mode =m;

  // ʤΤΤä˴
  if( mode == INPUT_MODE_HIRA || mode == INPUT_MODE_KATA){
    jrKanjiControl(contextID,KC_KILL,(char *)&ksv);
  }

  
  ksv.ks = &ks;
  ksv.buffer = (unsigned char *)buffer;
  ksv.bytes_buffer = BUF_SIZE;

  switch(mode){
  case INPUT_MODE_HIRA:
    ksv.val = CANNA_MODE_HenkanMode;
    break;
  case INPUT_MODE_KATA:
    ksv.val = CANNA_MODE_ZenKataKakuteiMode;
    break;
  case INPUT_MODE_WIDE:
    ksv.val = CANNA_MODE_ZenAlphaKakuteiMode;
    break;
  case INPUT_MODE_RAW:
    ksv.val = CANNA_MODE_AlphaMode;
    break;
  }
  jrKanjiControl(contextID,KC_CHANGEMODE,(char *)&ksv);
}

jstring_t *CannaContext::clear()
{
  // ƥȤ
  return 0;
}

==> jmode-0.4.24.tar.gz: jmode-0.4.24/src/cannaconv.cpp <==
// libcannaconv.so canna Ѥplugin
//
#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <canna/jrkanji.h>
#include "canna.xpm"
#include "cannaconv.h"

bool init_conv()
{
  // ⤷ǼԤ顢δؿfalse֤
  CannaConv *canna = new CannaConv();
  canna->init();
  register_kkconv(canna);
    
  return true;
}

CannaConv::~CannaConv()
{
  // λν
  jrKanjiControl(0,KC_FINALIZE,NULL);
}

KKContext *CannaConv::createContext(XimIC *ic)
{
  // ϥƥȤ롣
  CannaContext *canna;
  canna = new CannaContext(this,ic);
  return canna;
}

char **CannaConv::getIcon()
{
  // toolbar˽ФƤ륢֤
  return canna_xpm;
}

void CannaConv::onPushIcon()
{
  // toolbarβ줿˸ƤФ롣
}

void CannaConv::init()
{
  //Cannaץꥱ̾Ρ⡼ɾͤɽ
  jrKanjiControl(0,KC_INITIALIZE,NULL);
  jrKanjiControl(0,KC_SETAPPNAME,"HekeIM");
  jrKanjiControl(0,KC_SETMODEINFOSTYLE,(char *)1);  
}

/////////////////////////////////////
//
CannaContext::CannaContext(CannaConv *,XimIC *ic) : KKContext(ic)
{
  contextID = (int)this;
  
  ksv.val = CANNA_MODE_AlphaMode;
  ksv.buffer = (unsigned char *)buffer;
  ksv.bytes_buffer = BUF_SIZE;
  ksv.ks = &ks;

  jrKanjiControl(contextID,KC_INITIALIZE,0);
  //jrKanjiControl(contextID,KC_SETMODEINFOSTYLE,(char*)1);
  mode = INPUT_MODE_RAW;
}

CannaContext::~CannaContext()
{
  // ȤʤʤäƥȤ
  jrKanjiControl(contextID,KC_CLOSEUICONTEXT,(char*)&ksv);
}

void CannaContext::OnUpdatePe(pe_stat *pe)
{
  // preedit˸ƤФ롣
  pe->clear();
  /* 
     if ( mode != INPUD_MODE_HIRA && mode!= INPUT_MODE_KATA){
     return ;// Ѵ⡼ɤǤʤpreedit¸ߤʤ
     }
     pe->new_segment(PE_UNDERLINE);//饢饤
     jstring_t s;
     p = ߤΥץꥨǥåʸ
     str_to_jstring(buffer,&s);
     jstring_t::iterator i;
     for ( i = s.begin() ; i != s.end() ; i++){
     pe->push_cchar(*i);
     }
     */
}

int CannaContext::push_key(keyState *e)
{
  // 줿Τˡ
  if ( mode == INPUT_MODE_RAW || mode == INPUT_MODE_WIDE){
    //줿Τ⡼ڤؤʳä餽Τޤcommitؿ
    return proc_raw_mode(e);
  }
  int nbytes;
  //Ѥʿ̾Ҳ̾ʤΤǤcannaΥȥޥȥ
  nbytes = jrKanjiString(contextID,e->char_code(),buffer,BUF_SIZE,&ks);
  //ץꥨǥåȤξ֤ѲƤΤǹ
  if ( nbytes > 0){
    buffer[nbytes] = 0;
    //update_preedit();
    printf("%s.\n",buffer);
  }
  return COMMIT_RAW;
}

int CannaContext::proc_raw_mode(keyState *e)
{
  if ( e->is_bind_to(A_hira_mode)){
    set_mode(INPUT_MODE_HIRA);
    return UPDATE_MODE;
  }
  return COMMIT_RAW;
}

int CannaContext::get_mode()
{
  return mode;
}

void CannaContext::set_mode(int m)
{
  if ( mode == m){
    return ;
  }

  mode =m;

  // ʤΤΤä˴
  if( mode == INPUT_MODE_HIRA || mode == INPUT_MODE_KATA){
    jrKanjiControl(contextID,KC_KILL,(char *)&ksv);
  }

  
  ksv.ks = &ks;
  ksv.buffer = (unsigned char *)buffer;
  ksv.bytes_buffer = BUF_SIZE;

  switch(mode){
  case INPUT_MODE_HIRA:
    ksv.val = CANNA_MODE_HenkanMode;
    break;
  case INPUT_MODE_KATA:
    ksv.val = CANNA_MODE_ZenKataKakuteiMode;
    break;
  case INPUT_MODE_WIDE:
    ksv.val = CANNA_MODE_ZenAlphaKakuteiMode;
    break;
  case INPUT_MODE_RAW:
    ksv.val = CANNA_MODE_AlphaMode;
    break;
  }
  jrKanjiControl(contextID,KC_CHANGEMODE,(char *)&ksv);
}

jstring_t *CannaContext::clear()
{
  // ƥȤ
  return 0;
}
